# -*- coding: utf-8 -*-
"""Project metadata."""

package = 'humilis'
project = 'humilis'
version = '0.0.4'
description = 'Helps you deploy infrastructure in the AWS cloud'
authors = ['German Gomez-Herrero', 'Innovative Travel Ltd']
authors_string = ', '.join(authors)
emails = ['german@innovativetravel.eu']
license = 'MIT'
copyright = '2015 ' + authors_string
url = 'http://github.com/InnovativeTravel/humilis'
